export interface RelatedLink {
  label: string;
  url: string;
  isExternal: boolean;
  scLabel: string;
}

export interface AdditionalInfo {
  text: string;
  isActive: boolean;
}

import {Component, h, Prop, Watch, State, Listen, Element} from '@stencil/core';
import {ListItemVariants} from '../../utils/utils';

let keys = {
  enter: 'Enter',
  space: 'Space',
};

@Component({
  tag: 'hy-list-item',
  styleUrl: 'hy-list-item.scss',
  shadow: false,
})
export class HyListItem {
  @Element() el: HTMLElement;
  @State() isExpandedRelatedContent: boolean = false;
  @Prop() variant: ListItemVariants = ListItemVariants.default;
  @Prop() itemType?: string;
  @Prop() itemTitle?: string;
  @Prop() itemDescription?: string;
  @Prop() url?: string;
  @Prop() isExternal: boolean = false;
  @Prop() scLabel?: string;
  @Prop() itemImageUrl?: string;
  @Prop() itemImageAlt?: string;
  @Prop() relatedLinksBlockTitle?: string;
  private _relatedLinks: RelatedLink[];
  @Prop() relatedLinks?: RelatedLink[] | string;
  @Prop() imageLabel?: string;
  @Prop() outGoingUrlLabel?: string = 'Go to programme page';
  private _additionalInfo: AdditionalInfo[];
  @Prop() additionalInfo?: AdditionalInfo[] | string;
  @Prop() headerstyle: string = 'common';

  @State() _imageMinHeight: number = 0;
  @State() _imageMinWidth: number = 0;

  @Watch('relatedLinks')
  relatedLinksWatcher(newValue: RelatedLink[] | string) {
    if (typeof newValue === 'string') {
      this._relatedLinks = JSON.parse(newValue);
    } else {
      this._relatedLinks = newValue;
    }
  }

  @Watch('additionalInfo')
  additionalInfoWatcher(newValue: AdditionalInfo[] | string) {
    if (typeof newValue === 'string') {
      this._additionalInfo = JSON.parse(newValue);
    } else {
      this._additionalInfo = newValue;
    }
  }

  expandRelatedInfoPanel(target) {
    let contentSelector = `hy-list-item-wrapper__${this.variant}__related-info__content`;
    let targetButtonClass = 'hy-list-item-wrapper__degree__related-info__heading__button';
    let contentSection = document.querySelectorAll(`[aria-labelledBy="${target.id}"]`)[0] as HTMLElement;
    if (contentSection) {
      if (!this.isExpandedRelatedContent) {
        contentSection.classList.add(`${contentSelector}__is-open`);
        target.classList.add(`${targetButtonClass}__is-open`);
      } else {
        contentSection.classList.remove(`${contentSelector}__is-open`);
        target.classList.remove(`${targetButtonClass}__is-open`);
      }
      this.isExpandedRelatedContent = !this.isExpandedRelatedContent;
    }
  }

  @Listen('keydown')
  handleKeyDown(event: KeyboardEvent) {
    const key = event.code;
    let target = event.target as HTMLButtonElement;

    const targetElement = target.tagName.toLowerCase();
    const possibleTags = [targetElement].some((r) => ['svg', 'path', 'button', 'a'].indexOf(r) >= 0);

    if (
      target &&
      possibleTags &&
      !target.classList.contains('hy-list-item-wrapper__degree__related-info__link') &&
      [keys.enter, keys.space].includes(key)
    ) {
      if (targetElement !== 'button') {
        target = target.closest(`.hy-list-item-wrapper__${this.variant}__related-info__heading__button`);
      }
      this.expandRelatedInfoPanel(target);
      event.preventDefault();
    }
  }

  @Listen('click')
  handleClick(event) {
    let target = event.target as HTMLElement;
    const targetElement = target.tagName.toLowerCase();
    const possibleTags = [targetElement].some((r) => ['svg', 'path', 'button', 'a'].indexOf(r) >= 0);

    if (target && possibleTags && !target.classList.contains('hy-list-item-wrapper__degree__related-info__link')) {
      if (targetElement !== 'button') {
        target = target.closest(`.hy-list-item-wrapper__${this.variant}__related-info__heading__button`);
      }
      this.expandRelatedInfoPanel(target);
      event.preventDefault();
      event.stopImmediatePropagation();
    }
  }

  componentWillLoad() {
    this.relatedLinksWatcher(this.relatedLinks);
    this.additionalInfoWatcher(this.additionalInfo);
  }

  componentDidLoad() {
    let hyMainDiv = this.el.closest('.hy-main');
    if (hyMainDiv) {
      if (!hyMainDiv.classList.contains('with-sidebar')) {
        this.headerstyle = 'large';
      }
    }
    if (this.variant == ListItemVariants.degreeProgramme) {
      const image = document.querySelector('.hy-list-item__degree__image-container__image img') as any;

      if (image) {
        image.addEventListener('load', function () {
          const outputImageAspectRatio = 1;
          const imageHeight = image.naturalHeight;
          const imageWidth = image.naturalWidth;
          const inputImageAspectRatio = imageWidth / imageHeight;
          this._imageMinWidth = imageWidth;
          this._imageMinHeight = imageHeight;
          if (inputImageAspectRatio > outputImageAspectRatio) {
            this._imageMinWidth = imageHeight * outputImageAspectRatio;
          } else if (inputImageAspectRatio < outputImageAspectRatio) {
            this._imageMinHeight = imageWidth / outputImageAspectRatio;
          }
        });
      }
    }
  }

  render() {
    const classWrapperAttributes = [
      'hy-list-item-wrapper',
      `hy-list-item-wrapper__${this.variant}`,
      `hy-list-item-wrapper__${this.variant}__${this.headerstyle}`,
    ].join(' ');
    const classAttributes = ['hy-list-item', `hy-list-item__${this.variant}`].join(' ');
    const classInfoContainerAttributes = [
      `hy-list-item__${this.variant}__info-container`,
      `hy-list-item__${this.variant}__info-container__${this.headerstyle}`,
    ].join(' ');
    const classInfoContainerDataAttributes = [
      `hy-list-item__${this.variant}__info-container__data`,
      `hy-list-item__${this.variant}__info-container__data__${this.headerstyle}`,
    ].join(' ');
    const classInfoContainerHeader = [
      `hy-list-item__${this.variant}__info-container__header`,
      `hy-list-item__${this.variant}__info-container__header__${this.headerstyle}`,
    ].join(' ');
    const classInfoContainerTitle = [
      `hy-list-item__${this.variant}__info-container__title`,
      `hy-list-item__${this.variant}__info-container__title__${this.headerstyle}`,
    ].join(' ');
    const classInfoContainerDescription = [
      `hy-list-item__${this.variant}__info-container__description`,
      `hy-list-item__${this.variant}__info-container__description__${this.headerstyle}`,
    ].join(' ');
    const classInfoContainerLinkContainer = [`hy-list-item__${this.variant}__info-container__link-container`].join(' ');
    const classInfoContainerLinkIcon = [`hy-list-item__${this.variant}__info-container__link__icon`].join(' ');
    const classInfoContainerLink = [`hy-list-item__${this.variant}__info-container__link`].join(' ');

    const target = this.isExternal ? '_blank' : '_self';

    switch (this.variant) {
      case ListItemVariants.degreeProgramme: {
        const classImageContainerAttributes = [`hy-list-item__${this.variant}__image-container`].join(' ');
        const classImageContainerImage = [`hy-list-item__${this.variant}__image-container__image`].join(' ');

        const classInfoContainerAdditionalInfoAttributes = [
          `hy-list-item__${this.variant}__info-container__additional-info`,
          `hy-list-item__${this.variant}__info-container__additional-info__${this.headerstyle}`,
        ].join(' ');

        const classInfoContainerOutgoingLink = [
          `hy-list-item__${this.variant}__info-container__outgoing-link`,
          `hy-list-item__${this.variant}__info-container__outgoing-link__${this.headerstyle}`,
        ].join(' ');
        const classInfoContainerOutgoingLinkUrl = [
          `hy-list-item__${this.variant}__info-container__outgoing-link__link`,
        ].join(' ');

        const classRelatedInfoContainer = [
          `hy-list-item-wrapper__${this.variant}__related-info`,
          `hy-list-item-wrapper__${this.variant}__related-info__${this.headerstyle}`,
        ].join(' ');
        const classRelatedInfoHeading = [`hy-list-item-wrapper__${this.variant}__related-info__heading`].join(' ');
        const classRelatedInfoHeadingLink = [`hy-list-item-wrapper__${this.variant}__related-info__heading__link`].join(
          ' '
        );
        const classRelatedInfoHeadingButton = [
          `hy-list-item-wrapper__${this.variant}__related-info__heading__button`,
        ].join(' ');
        const classRelatedInfoHeadingIcon = [`hy-list-item-wrapper__${this.variant}__related-info__heading__icon`].join(
          ' '
        );
        const classRelatedInfoContent = [`hy-list-item-wrapper__${this.variant}__related-info__content`].join(' ');
        const classRelatedInfoLink = [`hy-list-item-wrapper__${this.variant}__related-info__link`].join(' ');

        const titleAsId =
          this.relatedLinksBlockTitle && this.relatedLinksBlockTitle.length > 0
            ? `${this.itemTitle}-${this.relatedLinksBlockTitle}`.toLowerCase().replace(/\W/g, '-')
            : '';
        const accordionItemHref = '#' + titleAsId + '--title';

        const relatedLinksCount =
          this.relatedLinksBlockTitle && this.relatedLinksBlockTitle.length > 0 ? this._relatedLinks.length : '';

        return [
          <article class={classWrapperAttributes}>
            <a class={classAttributes} href={this.url} target={target} aria-label={this.scLabel}>
              <div class={classImageContainerAttributes}>
                <div class={classImageContainerImage}>
                  <img
                    aria-hidden="true"
                    alt={this.itemImageAlt}
                    src={this.itemImageUrl}
                    style={{
                      '--minheight': `${this._imageMinHeight}px` as 'minHeight',
                      '--minwidth': `${this._imageMinWidth}px` as 'minWidth',
                    }}
                  />
                </div>
                {this.imageLabel && <span>{this.imageLabel}</span>}
              </div>
              <div class={classInfoContainerAttributes}>
                <div class={classInfoContainerDataAttributes}>
                  <div class={classInfoContainerTitle}>{this.itemTitle}</div>
                  {this.itemDescription && <div class={classInfoContainerDescription}>{this.itemDescription}</div>}
                  {this._additionalInfo && (
                    <div class={classInfoContainerAdditionalInfoAttributes}>
                      {this._additionalInfo.map((x) => {
                        let classInfoContainerAdditionalInfoItem = [
                          `hy-list-item__${this.variant}__info-container__additional-info__item`,
                          `hy-list-item__${this.variant}__info-container__additional-info__item__${this.headerstyle}`,
                          `${x.isActive.toString() == 'true' ? 'active' : ''}`,
                        ].join(' ');
                        return <div class={classInfoContainerAdditionalInfoItem}>{x.text}</div>;
                      })}
                    </div>
                  )}
                </div>
                <div class={classInfoContainerOutgoingLink}>
                  <a
                    href={this.url}
                    target={target}
                    aria-label={this.scLabel}
                    class={classInfoContainerOutgoingLinkUrl}
                    tabindex="-1"
                  >
                    <span class="label">{this.outGoingUrlLabel}</span>
                    <span class="icon-wrapper">
                      <div class="icon">
                        <hy-icon icon={'hy-icon-arrow-to-right'} size={32} />
                      </div>
                    </span>
                  </a>
                </div>
              </div>
            </a>
            {this.relatedLinksBlockTitle && (
              <div class={classRelatedInfoContainer}>
                <div class={classRelatedInfoHeading}>
                  <a href={accordionItemHref} class={classRelatedInfoHeadingLink} tabindex="-1">
                    <button
                      aria-expanded="false"
                      aria-controls={`${titleAsId}--content`}
                      class={classRelatedInfoHeadingButton}
                      id={`${titleAsId}--title`}
                    >
                      <span class={classRelatedInfoHeadingIcon}>
                        <hy-icon icon={'hy-icon-caret-down'} size={14} />
                      </span>
                      {this.relatedLinksBlockTitle} ({relatedLinksCount})
                    </button>
                  </a>
                </div>
                <div
                  class={classRelatedInfoContent}
                  aria-labelledBy={`${titleAsId}--title`}
                  id={`${titleAsId}--content`}
                  role="region"
                  aria-hidden="true"
                >
                  {this._relatedLinks.map((x) => {
                    let relatedLinkTarget = x.isExternal ? '_blank' : '_self';
                    return (
                      <a class={classRelatedInfoLink} href={x.url} target={target} aria-label={relatedLinkTarget}>
                        {x.label}
                      </a>
                    );
                  })}
                </div>
              </div>
            )}
          </article>,
        ];
      }
      default: {
        return (
          <article>
            <a class={classAttributes} href={this.url} target={target} aria-label={this.scLabel}>
              <div class={classInfoContainerAttributes}>
                <div class={classInfoContainerHeader}>{this.itemType}</div>
                <div class={classInfoContainerTitle}>{this.itemTitle}</div>

                <div class={classInfoContainerLinkContainer}>
                  <span class={classInfoContainerLinkIcon}>
                    <hy-icon icon={'hy-icon-link'} size={15} />
                  </span>
                  <div class={classInfoContainerLink}>{this.url}</div>
                </div>

                {this.itemDescription && <div class={classInfoContainerDescription}>{this.itemDescription}</div>}
              </div>
            </a>
          </article>
        );
      }
    }
  }
}
